// $Id: stats.C,v 1.2 2001/04/12 14:55:57 dvermeir Exp $
#include "stats.h"
#include <algorithm> // min()

// We copy the configuration into configuration_.
Stats::Stats(const Configuration& c): configuration_(c) {
}

// Update the internal database with information from a LogRecord.
void
Stats::add(const LogRecord& r) {
// What we do depends on the requested configuration format.
/* It would be more efficient to make these actual data members
   of Stat. That way, they would only be ``computed'' once.
*/
set<unsigned int> levels(configuration_.levels());
unsigned int      max_level(configuration_.max_level());

switch (configuration_.format()) {

  case Configuration::DATE: {
    const DatePattern&  d(r.date());
    // We will have at most 4 components in a date vector.
    // This should become a symbolic constant in Configuration!
    unsigned int m(min(d.size(),max_level));
    for (unsigned int l=1;(l<=m);++l) 
      if (levels.count(l)) {
        DatePattern dp(d,l);
        ++dates_[dp];
        }
    }
    break;

  case Configuration::PATH: {
    const Path& path(r.path());
    unsigned int m(min(path.size(),max_level));
    for (unsigned int l=1; l<=m; ++l) 
      if (levels.count(l)) {
        Path pp(path,l);
        ++paths_[pp];
        }
    }
    break;

  case Configuration::DOMAIN: {
    const Domain& domain(r.domain());
    unsigned int sz(domain.size());
    unsigned int m(min(sz,max_level));
    for (unsigned int l=1; l<=m; ++l) {
      if (levels.count(l)) {
        Domain dom(domain,l);
        ++(domains_[dom]);
        }
      }
    }
    break;
  }
}

// Print dates_ map to os.
void
Stats::print_dates(ostream& os) const {
for (DateCount::const_iterator i=dates_.begin(); i!=dates_.end(); ++i) {
  const DatePattern& d((*i).first);
  os << d <<  "\t" << (*i).second << "\n";
  }
}

// Print paths_ map to os.
void
Stats::print_paths(ostream& os) const {
for (PathCount::const_iterator i=paths_.begin(); i!=paths_.end(); ++i) {
  const Path& path((*i).first);
  os << path << "\t" << (*i).second << "\n";
  }
}

// Print domains_ map to os.
void
Stats::print_domains(ostream& os) const {
for (DomainCount::const_iterator i=domains_.begin(); i!=domains_.end(); ++i) {
  const Domain& domain((*i).first);
  os << domain <<  "\t" << (*i).second << "\n";
  }
}

// Print internal database, depending on configuration.
ostream&
operator<<(ostream& os,const Stats& stats) {
switch (stats.configuration_.format()) {
  case Configuration::DATE:
    stats.print_dates(os);
    break;
  case Configuration::PATH:
    stats.print_paths(os);
    break;
  case Configuration::DOMAIN:
    stats.print_domains(os);
    break;
  }
return os;
}
